#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <stdlib.h>
#include "global.h"
#include "rf12.h"

#define F_CPU 1000000
#include <util/delay.h>

#define SDI		5
#define SCK		6
#define CS		7
#define SDO		0
#define IRQ		1
#define RES		4

unsigned short rf12_trans(unsigned short wert)
{	unsigned short werti=0;
	unsigned char i;

	cbi(PORTB, CS);
	_delay_us(1);
	for (i=0; i<16; i++)
	{	if (wert&32768)
			sbi(PORTB, SDI);
		else
			cbi(PORTB, SDI);
		werti<<=1;
		if (PINB&(1<<SDO))
			werti|=1;
		sbi(PORTB, SCK);
		wert<<=1;
		_delay_us(0.3);
		cbi(PORTB, SCK);
	}
	sbi(PORTB, CS);
	return werti;
}

void rf12_init(void)
{	unsigned char i;
	DDRB=(1<<SDI)|(1<<SCK)|(1<<CS);
	PORTB=(1<<CS)|(1<<RES);

	for (i=0; i<50; i++)
		_delay_ms(10);

	rf12_trans(0xC0E0);		// AVR CLK: 10MHz
	rf12_trans(0x80D7);		// Enable FIFO
	rf12_trans(0xC2AC);		// Data Filter
	rf12_trans(0xCA81);		// FIFO mode
	rf12_trans(0xE000);		// disable wakeuptimer
	rf12_trans(0xC800);		// disable low duty cycle
//	rf12_trans(0xC4F7);		// AFC settings

}

void rf12_mode(unsigned char tx_on, unsigned char rx_on)
{
	if (rx_on)
	{	rf12_trans(0xCA81);		// set FIFO mode
		if (tx_on)
			rf12_trans(0x82F8);
		else
			rf12_trans(0x82C8);
		rf12_trans(0xCA83);		// enable FIFO
	}
	else
	{	rf12_trans(0xCA81);		// disable FIFO
		if (tx_on)
			rf12_trans(0x8238);
		else
			rf12_trans(0x8208);
	}
	_delay_ms(1);
}

void rf12_setbandwidth(unsigned char bandwidth, unsigned char gain, unsigned char drssi)
{
	rf12_trans(0x9400|((bandwidth&7)<<5)|((gain&3)<<3)|(drssi&7));
}

void rf12_setfreq(unsigned short freq)
{	if (freq<36)		// 430,0900MHz
		freq=36;
	else if (freq>3903)	// 439,7575MHz
		freq=3903;
	rf12_trans(0xA000|freq);
}

void rf12_setbaud(unsigned short baud)
{
	if (baud<663)
		return;
	if (baud<5400)					// Baudrate= 344827,58621/(R+1)/(1+CS*7)
		rf12_trans(0xC680|((43104/baud)-1));
	else
		rf12_trans(0xC600|((344828UL/baud)-1));
}

void rf12_setpower(unsigned char power, unsigned char mod)
{	
	rf12_trans(0x9800|(power&7)|((mod&15)<<4));
}

void rf12_tx(unsigned char byte)
{	while(PINB&(1<<IRQ));			//wait for previously TX over
	rf12_trans(0xB800|byte);
}

unsigned char rf12_status(void)
{
	return rf12_trans(0x0000);
}

unsigned char rf12_rx(void)
{	while(PINB&(1<<IRQ));			//wait until data in buffer
	return rf12_trans(0xB000);
}
